//
//  tank
//
//  Import and Playlist Daemon for autoradio project
//
//
//  Copyright (C) 2017-2019 Christian Pointner <equinox@helsinki.at>
//
//  This file is part of tank.
//
//  tank is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  any later version.
//
//  tank is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with tank. If not, see <http://www.gnu.org/licenses/>.
//

package auth

import (
	"crypto/rand"
	"encoding/base64"
	"encoding/json"
	"net/http"
)

type contextKey int

var (
	sessionContextKey contextKey
)

func generateRandomString(len int) (string, error) {
	b := make([]byte, len)
	if _, err := rand.Read(b[:]); err != nil {
		return "", err
	}
	return base64.RawURLEncoding.EncodeToString(b[:]), nil
}

type AuthBackendInfo struct {
	Name        string
	Description string
}

type NewSessionRequest struct {
	Backend   string          `json:"backend"`
	Arguments json.RawMessage `json:"arguments"`
}

type NewSessionResponse struct {
	Session *Session `json:"session"`
	Token   string   `json:"token"`
}

func sendHTTPResponse(w http.ResponseWriter, status int, resp interface{}) {
	w.Header().Set("Content-Type", "application/json")
	w.WriteHeader(status)
	json.NewEncoder(w).Encode(resp) // TODO: Error Handling?
}

type HTTPErrorResponse struct {
	Error string `json:"error,omitempty"`
}

func sendHTTPErrorResponse(w http.ResponseWriter, status int, error string) {
	sendHTTPResponse(w, status, HTTPErrorResponse{Error: error})
}

func sendHTTPInvalidSessionResponse(w http.ResponseWriter) {
	sendHTTPErrorResponse(w, http.StatusUnauthorized, "Request does not contain a valid token or session is already expired.")
}
